@@ -64,7 +64,7 @@
 		"\nGPE File Manager\n\ndctanner@magenet.com"\
 		"\nflorian@handhelds.org"
 
-#define NOHELPMESSAGE N_("Displaying help failed.")
+#define NOHELPMESSAGE N_("Help for this application is not installed.")
 
 
 /* IDs for data storage fields */
@@ -88,6 +88,7 @@ enum
 	COL_DIRDATA,
 	N_DIRCOLUMNS
 };
+
 static GtkTreeStore *dirstore = NULL;
 
 static GtkWidget *window;
@@ -96,9 +97,11 @@ static GtkWidget *view_widget;
 static GtkWidget *dir_view_widget;
 static GtkWidget *view_window;
 static GtkWidget *dir_view_window;
+static GtkWidget *active_view;
 static GtkWidget *vbox, *main_paned;
 
 static GtkWidget *bluetooth_menu_item;
+static GtkWidget *irda_menu_item;
 static GtkWidget *copy_menu_item;
 static GtkWidget *paste_menu_item;
 static GtkWidget *open_menu_item;
@@ -134,19 +137,20 @@ gchar *current_directory = NULL;
 static GnomeVFSMonitorHandle *dir_monitor = NULL;
 static gboolean refresh_scheduled = FALSE;
 
-gchar *current_view = "icons";
 gint current_zoom = 36;
 static gboolean view_is_icons = FALSE;
 static gboolean directory_browser = TRUE;
 static gboolean limited_view = FALSE;
 
-static gchar *file_clipboard = NULL;
+static GList *file_clipboard = NULL;
 
 static GHashTable *loaded_icons;
 static GtkWidget *progress_dialog = NULL;
 static gboolean abort_transfer = FALSE;
+static GList *dirlist = NULL;
 static gboolean initialized = FALSE;
 
+
 typedef struct
 {
   gchar *filename;
@@ -185,6 +189,7 @@ static void popup_ask_delete_file (void);
 static void show_file_properties (void);
 static void refresh_current_directory (void);
 static void send_with_bluetooth (void);
+static void send_with_irda (void);
 static void copy_file_clip (void);
 static void paste_file_clip (void);
 static void create_directory_interactive(void);
@@ -194,6 +199,7 @@ void on_about_clicked (GtkWidget * w);
 void on_help_clicked (GtkWidget * w);
 void on_dirbrowser_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data);
 void on_myfiles_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data);
+void do_select_all(GtkWidget *w, gpointer d);
 
 
 /* items of the context menu */
@@ -202,11 +208,12 @@ static GtkItemFactoryEntry menu_items[] =
   { "/Open Wit_h",	 NULL, popup_ask_open_with,  0, "<StockItem>", GTK_STOCK_OPEN },
   { "/sep1",	         NULL, NULL,	             0, "<Separator>" },
   { "/Send via _Bluetooth", NULL, send_with_bluetooth, 0, "<Item>" },
+  { "/Send via _Infrared", NULL, send_with_irda, 0, "<Item>" },
   { "/_Copy",          NULL, copy_file_clip,         0, "<StockItem>", GTK_STOCK_COPY },
   { "/_Paste",          NULL, paste_file_clip,         0, "<StockItem>", GTK_STOCK_PASTE },
   { "/_Move",            NULL, popup_ask_move_file,            0, "<Item>" },
   { "/_Rename",          NULL, popup_ask_rename_file,          0, "<Item>" },
-  { "/_Delete",          NULL, popup_ask_delete_file,         0, "<StockItem>", GTK_STOCK_DELETE },
+  { "/_Delete",   "Delete", popup_ask_delete_file,         0, "<StockItem>", GTK_STOCK_DELETE },
   { "/_Create Directory",NULL, create_directory_interactive, 0, "<Item>"},
   { "/sep2",	         NULL, NULL,	             0, "<Separator>" },
   { "/_Properties",      NULL, show_file_properties, 0, "<StockItem>", GTK_STOCK_PROPERTIES },
@@ -217,8 +224,15 @@ static int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 /* items of the main menu */
 static GtkItemFactoryEntry mMain_items[] = {
   { N_("/_File"),         NULL,         NULL, 0, "<Branch>" },
+  { N_("/_File/_Copy"),          NULL, copy_file_clip,         0, "<StockItem>", GTK_STOCK_COPY },
+  { N_("/_File/_Paste"),          NULL, paste_file_clip,         0, "<StockItem>", GTK_STOCK_PASTE },
+  { N_("/_File/_Move"),            NULL, popup_ask_move_file,            0, "<Item>" },
+  { N_("/_File/_Rename"),          NULL, popup_ask_rename_file,          0, "<Item>" },
+  { N_("/_File/_Delete"),   "Delete", popup_ask_delete_file,         0, "<StockItem>", GTK_STOCK_DELETE },
   { N_("/_File/_Create Directory"),NULL, create_directory_interactive, 0, "<Item>"},
   { N_("/_File/s1"), NULL , NULL,    0, "<Separator>"},
+  { N_("/_File/Select _All"),"Control+A", do_select_all, 0, "<Item>"},
+  { N_("/_File/s2"), NULL , NULL,    0, "<Separator>"},
   { N_("/_File/_Close"),  NULL, gtk_main_quit, 0, "<StockItem>", GTK_STOCK_QUIT },
   { N_("/_Settings"),         NULL,         NULL, 0, "<Branch>" },
   { N_("/_Settings/Directory Browser"), NULL, on_dirbrowser_setting_changed, 0, "<CheckItem>"},
@@ -303,7 +317,7 @@ on_myfiles_setting_changed(GtkCheckMenuItem *menuitem, gpointer user_data)
 void
 on_about_clicked (GtkWidget * w)
 {
-	show_message(GTK_MESSAGE_INFO,HELPMESSAGE);
+	show_message(GTK_MESSAGE_INFO, HELPMESSAGE);
 }
 
 
@@ -311,7 +325,7 @@ void
 on_help_clicked (GtkWidget * w)
 {
 	if (gpe_show_help(PACKAGE,NULL))
-		show_message(GTK_MESSAGE_ERROR,NOHELPMESSAGE);
+		show_message(GTK_MESSAGE_INFO, NOHELPMESSAGE);
 }
 
 
@@ -567,6 +581,15 @@ create_directory_interactive(void)
   gtk_widget_destroy(dialog);
 }
 
+void
+do_select_all(GtkWidget *w, gpointer d)
+{
+  GtkTreeSelection *sel;
+  
+  sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(active_view));
+  gtk_tree_selection_select_all(sel);
+}
+
 static void
 copy_file (const gchar *src_uri_txt, const gchar *dest_uri_txt)
 {
@@ -600,34 +623,86 @@ copy_file (const gchar *src_uri_txt, const gchar *dest_uri_txt)
 }
 
 static void 
-copy_file_clip (void)
+clear_clipboard (void)
 {
-  if (file_clipboard) 
-    g_free(file_clipboard);
-  if (GNOME_VFS_FILE_INFO_LOCAL(current_popup_file->vfs))
-    file_clipboard = 
-      gnome_vfs_get_uri_from_local_path(current_popup_file->filename);
-  else
-    file_clipboard = g_strdup(current_popup_file->filename);
-  gpe_popup_infoprint(GDK_DISPLAY(), _("File copied to clipboard."));
+  GList *iter = file_clipboard;
+  
+  if (!file_clipboard) 
+    return;
+  
+  while (iter)
+  {
+    g_free(iter->data);
+    iter = iter->next;
+  }
+  g_list_free(file_clipboard);
+  file_clipboard = NULL;
 }
 
+static void
+clip_one_file (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+               gpointer data)
+{
+  FileInformation *cfi = NULL;
+  gint col = (gint)data;
+
+  gtk_tree_model_get(model, iter, col, &cfi, -1);
+  if (cfi)
+  {
+    if (GNOME_VFS_FILE_INFO_LOCAL(cfi->vfs))
+      file_clipboard = g_list_append(file_clipboard, 
+                                     gnome_vfs_get_uri_from_local_path(cfi->filename));
+    else
+      file_clipboard = g_list_append(file_clipboard, g_strdup(cfi->filename));
+  }
+}
+
+static void 
+copy_file_clip (void)
+{
+  clear_clipboard();
+  GtkTreeSelection *sel = NULL;
+  gint col;
+
+  if (active_view)
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(active_view));
+  if (active_view == view_widget)
+    col = COL_DATA;
+  else
+    col = COL_DIRDATA;
+  
+  if (!sel) 
+    return;
+  
+  gtk_tree_selection_selected_foreach(sel, 
+                                     (GtkTreeSelectionForeachFunc)clip_one_file,
+                                     (gpointer)col);
+  
+  gpe_popup_infoprint(GDK_DISPLAY(), _("File(s) copied to clipboard."));
+}
+
+
+static void
+copy_one_file (gpointer file, gpointer userdata)
+{
+  gchar *target_file;
+  target_file = 
+    g_strdup_printf("%s/%s",
+                    current_directory,
+                    g_path_get_basename(file));
+  if (strcmp(file, target_file)) /* check if not the same */
+    copy_file(file, target_file);
+  g_free(target_file);
+}
 
 static void 
 paste_file_clip (void)
 {
-  gchar *target_file;
-  
   if (file_clipboard == NULL) 
     return;
   
-  target_file = 
-    g_strdup_printf("%s/%s",
-                    current_directory,
-                    g_path_get_basename(file_clipboard));
-  if (strcmp(file_clipboard,target_file)) /* check if not the same */
-    copy_file(file_clipboard,target_file);
-  g_free(target_file);
+  g_list_foreach(file_clipboard, copy_one_file, NULL);
+  
   refresh_current_directory(); 
 }
 
@@ -759,64 +834,84 @@ rename_file (GtkWidget *dialog_window, gint response_id)
   refresh_current_directory();
 }
 
+
 static void
-move_file (gchar *directory)
+move_one_file (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+               gpointer data)
 {
-  gchar *dest, *error;
   GnomeVFSResult result;
+  gint col;
+  gchar *directory = (gchar*)data;
+  gchar *dest;
 
-  dest = g_strdup_printf ("%s/%s", directory, current_popup_file->vfs->name);
-
-  result = gnome_vfs_move_uri (gnome_vfs_uri_new (current_popup_file->filename), gnome_vfs_uri_new (dest), TRUE);
-
-  if (result != GNOME_VFS_OK)
-    {
-      switch (result)
-      {
-      case GNOME_VFS_ERROR_NO_SPACE:
-        error = g_strdup ("No space left on device.");
-        break;
-      case GNOME_VFS_ERROR_READ_ONLY:
-        error = g_strdup ("Destination is read only.");
-        break;
-      case GNOME_VFS_ERROR_ACCESS_DENIED:
-        error = g_strdup ("Access denied.");
-        break;
-      case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
-        error = g_strdup ("Read only file system.");
-        break;
-      default:
-        error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
-        break;
-      }
-      gpe_error_box (error);
-      g_free (error);
-    }
+  if (active_view == view_widget)
+    col = COL_DATA;
   else
+    col = COL_DIRDATA;
+
+  FileInformation *cfi = NULL;
+  
+  gtk_tree_model_get(model, iter, col, &cfi, -1);
+  if (cfi)
     {
-      gpe_popup_infoprint(GDK_DISPLAY(), _("File moved."));
-      refresh_current_directory();
+      dest = g_strdup_printf ("%s/%s", directory, cfi->vfs->name);
+      result = gnome_vfs_move_uri (gnome_vfs_uri_new (cfi->filename), 
+                                   gnome_vfs_uri_new (dest), TRUE);
+      g_free (dest); 
+      if (result != GNOME_VFS_OK)
+        gpe_error_box (gnome_vfs_result_to_string (result));
     }
-  g_free (dest);
+}
+
+static void
+move_files (gchar *directory)
+{
+  GtkTreeSelection *sel = NULL;
+  
+  if (active_view)
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(active_view));
+  
+  if (!sel) 
+    return;
+  
+  if (gtk_tree_selection_count_selected_rows(sel))
+  {
+    gtk_tree_selection_selected_foreach(sel, 
+                                        (GtkTreeSelectionForeachFunc)move_one_file,
+                                        (gpointer)directory);
+    refresh_current_directory();
+    gpe_popup_infoprint(GDK_DISPLAY(), _("File(s) moved."));
+  }
 }
 
 static void
 popup_ask_move_file ()
 {
   GtkWidget *dirbrowser_window;
+  GtkTreeSelection *sel = NULL;
 
-  dirbrowser_window = gpe_create_dir_browser (_("Move to directory..."), 
-    (gchar *) g_get_home_dir (), GTK_SELECTION_SINGLE, move_file);
-  gtk_window_set_transient_for (GTK_WINDOW (dirbrowser_window), GTK_WINDOW (window));
+  if (active_view)
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(active_view));
+  
+  if (!sel) 
+    return;
+  
+  if (gtk_tree_selection_count_selected_rows(sel))
+  {
+    dirbrowser_window = gpe_create_dir_browser (_("Move to directory..."), 
+                                                (gchar *) g_get_home_dir (), 
+                                                GTK_SELECTION_SINGLE, move_files);
+    gtk_window_set_transient_for (GTK_WINDOW (dirbrowser_window), GTK_WINDOW (window));
 
-  gtk_widget_show_all (dirbrowser_window);
+    gtk_widget_show_all (dirbrowser_window);
+  }
 }
 
 static void
 popup_ask_rename_file ()
 {
   GtkWidget *dialog_window;
-  GtkWidget *vbox, *label, *entry;
+  GtkWidget *vbox, *label, *entry, *btnok;
   gchar *label_text;
 
   label_text = g_strdup_printf ("Rename file %s to:", current_popup_file->vfs->name);
@@ -826,9 +921,11 @@ popup_ask_rename_file ()
                                                GTK_DIALOG_MODAL 
                                                  | GTK_DIALOG_DESTROY_WITH_PARENT, 
                                                GTK_STOCK_CANCEL, 
-                                               GTK_RESPONSE_REJECT, 
-                                               GTK_STOCK_OK, 
-                                               GTK_RESPONSE_ACCEPT, NULL);
+                                               GTK_RESPONSE_REJECT, NULL);
+  btnok = gtk_dialog_add_button(GTK_DIALOG(dialog_window), 
+                                GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
+  GTK_WIDGET_SET_FLAGS(btnok, GTK_CAN_DEFAULT);
+  gtk_widget_grab_default(btnok);
   g_signal_connect (G_OBJECT (dialog_window), "delete_event", 
                     G_CALLBACK (kill_widget), dialog_window);
   g_signal_connect (G_OBJECT (dialog_window), "response", 
@@ -837,9 +934,11 @@ popup_ask_rename_file ()
   vbox = gtk_vbox_new (FALSE, 0);
 
   label = gtk_label_new (label_text);
+  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
   g_free (label_text);
 
   entry = gtk_entry_new ();
+  gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
   g_object_set_data (G_OBJECT (dialog_window), "entry", entry);
 
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog_window)->vbox), vbox);
@@ -849,27 +948,131 @@ popup_ask_rename_file ()
   gtk_widget_show_all (dialog_window);
 }
 
-static void
-popup_ask_delete_file ()
+gboolean 
+delete_cb_recursive(const gchar *rel_path,
+                   GnomeVFSFileInfo *info,
+                   gboolean recursing_will_loop,
+                   gpointer data,
+                   gboolean *recurse)
 {
-  gchar *label_text;
+   gchar *path = data;
+   gchar *fn = g_strdup_printf("%s/%s", path, rel_path);
+  
+   if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+   {
+     *recurse = TRUE;
+     GnomeVFSResult r = 0;
+     if (!recursing_will_loop)
+     r = gnome_vfs_directory_visit (fn,
+                                GNOME_VFS_FILE_INFO_DEFAULT,
+                                GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
+                                (GnomeVFSDirectoryVisitFunc) delete_cb_recursive,
+                                strdup(fn)); 
+     /* HACK: We collect the empty directories in a list to delete after 
+              visiting the complete tree to prevent gnome-vfs from stopping
+              visiting directories 
+     */
+     if (r == GNOME_VFS_OK)
+     {
+       dirlist = g_list_append(dirlist, fn);
+     }
+     else
+       fprintf(stderr, "Err %s\n", gnome_vfs_result_to_string(r));
 
-  label_text = g_strdup_printf (_("Delete \"%s\"?"), current_popup_file->vfs->name);
+   }
+   else
+   {
+     gnome_vfs_unlink_from_uri (gnome_vfs_uri_new (fn));
+     g_free(fn);
+   }
+   
+   return TRUE;
+}
 
-  if (gpe_question_ask (label_text, _("Confirm"), "!gtk-dialog-question", 
-			"!gtk-cancel", NULL, "!gtk-delete", NULL, NULL, NULL) == 1)
+
+static GnomeVFSResult
+delete_directory_recursive(FileInformation *dir)
+{
+  gchar *path = g_strdup(dir->filename);
+  GList *iter;
+  gnome_vfs_directory_visit    (dir->filename,
+                                GNOME_VFS_FILE_INFO_DEFAULT,
+                                GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
+                                (GnomeVFSDirectoryVisitFunc) delete_cb_recursive,
+                                path); 
+  g_free(path);
+  iter = dirlist;
+  while (iter)
     {
-      GnomeVFSResult r;
+      gnome_vfs_remove_directory(iter->data);
+      g_free(iter->data);
+      iter=iter->next;
+    }
+  g_list_free(dirlist);
+  dirlist = NULL;    
+  return gnome_vfs_remove_directory_from_uri(gnome_vfs_uri_new (dir->filename));
+}
 
-      r = gnome_vfs_unlink_from_uri (gnome_vfs_uri_new (current_popup_file->filename));
+
+static void
+delete_one_file (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+                 gpointer data)
+{
+  gboolean isdir;
+  GnomeVFSURI *uri;
+  GnomeVFSResult r;
+  gint col = (gint)data;
+
+  FileInformation *cfi = NULL;
+  
+  gtk_tree_model_get(model, iter, col, &cfi, -1);
+  if (cfi)
+  {
+    isdir = (cfi->vfs->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
+      if (isdir)
+        r = delete_directory_recursive(cfi);
+      else
+        {
+          uri = gnome_vfs_uri_new (cfi->filename);
+          r = gnome_vfs_unlink_from_uri (uri);
+          gnome_vfs_uri_unref(uri);
+        }
 
       if (r != GNOME_VFS_OK)
         gpe_error_box (gnome_vfs_result_to_string (r));
-      
-      refresh_current_directory();
-    }
+  }
+}
 
-  g_free (label_text);
+
+static void
+popup_ask_delete_file ()
+{
+  GtkTreeSelection *sel = NULL;
+  gint col;
+  
+  if (active_view)
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(active_view));
+  if (active_view == view_widget)
+    col = COL_DATA;
+  else
+    col = COL_DIRDATA;
+  
+  if (!sel) 
+    return;
+  
+  if (gtk_tree_selection_count_selected_rows(sel))
+  {
+    if (gpe_question_ask (_("Delete selected object(s)?"), 
+                          _("Confirm"), "!gtk-dialog-question", 
+	                     "!gtk-cancel", NULL, "!gtk-delete", NULL, 
+                         NULL, NULL) == 1)
+      {
+        gtk_tree_selection_selected_foreach(sel, 
+                                            (GtkTreeSelectionForeachFunc)delete_one_file,
+                                            (gpointer)col);
+        refresh_current_directory();
+      }
+  }
 }
 
 static void
@@ -1064,6 +1267,12 @@ send_with_bluetooth (void)
   bluetooth_send_file (current_popup_file->filename, current_popup_file->vfs);
 }
 
+static void
+send_with_irda (void)
+{
+  irda_send_file (current_popup_file->filename, current_popup_file->vfs);
+}
+
 static void
 popup_ask_open_with ()
 {
@@ -1082,6 +1291,7 @@ show_popup (GtkWidget *widget, gpointer udata)
     gtk_widget_set_sensitive(open_menu_item,FALSE);
     gtk_widget_set_sensitive(copy_menu_item,FALSE);
     gtk_widget_set_sensitive(bluetooth_menu_item,FALSE);
+    gtk_widget_set_sensitive(irda_menu_item,FALSE);
     gtk_widget_set_sensitive(move_menu_item,FALSE);
     gtk_widget_set_sensitive(rename_menu_item,FALSE);
     gtk_widget_set_sensitive(properties_menu_item,FALSE);
@@ -1102,6 +1312,10 @@ show_popup (GtkWidget *widget, gpointer udata)
       gtk_widget_set_sensitive (bluetooth_menu_item, TRUE);
     else
       gtk_widget_set_sensitive (bluetooth_menu_item, FALSE);
+    if (irda_available ())
+      gtk_widget_set_sensitive (irda_menu_item, TRUE);
+    else
+      gtk_widget_set_sensitive (irda_menu_item, FALSE);
   }
   if (file_clipboard)
     gtk_widget_set_sensitive (paste_menu_item, TRUE);
@@ -1281,7 +1495,9 @@ add_icon (FileInformation *file_info)
   else
     { /* add to iconlist or normal browser */
 	  if (view_is_icons)
-		gpe_icon_list_view_add_item_pixbuf (GPE_ICON_LIST_VIEW (view_widget), file_info->vfs->name, pixbuf, (gpointer) file_info);
+		gpe_icon_list_view_add_item_pixbuf (GPE_ICON_LIST_VIEW (view_widget), 
+                                            file_info->vfs->name, pixbuf, 
+                                            (gpointer) file_info);
 	  else
 	  {
 		GtkTreeIter iter;
@@ -1324,7 +1540,7 @@ compare_file_info (FileInformation *a, FileInformation *b)
 
 /* Render the contents for the current directory. */
 static void
-make_view ()
+make_view (void)
 {
   GnomeVFSDirectoryHandle *handle = NULL;
   GnomeVFSFileInfo *vfs_file_info = NULL;
@@ -1361,7 +1577,9 @@ make_view ()
 
   open_dir_result = 
     gnome_vfs_directory_open (&handle, current_directory, 
-                              GNOME_VFS_FILE_INFO_DEFAULT);
+                              GNOME_VFS_FILE_INFO_DEFAULT 
+                                | GNOME_VFS_FILE_INFO_FOLLOW_LINKS 
+                                | GNOME_VFS_FILE_INFO_GET_MIME_TYPE );
 
   while (open_dir_result == GNOME_VFS_OK)
   {
@@ -1474,11 +1692,15 @@ directory_changed (GnomeVFSMonitorHandle *handle,
 void
 browse_directory (gchar *directory)
 {
-  gboolean enabled;
+  gboolean enabled, ishome;
+  gchar *msg;
+  
+  ishome = !strcmp(directory, g_get_home_dir());
+  
   /* disable "up" button if we are in homedir and view is limited */
-  enabled = (!limited_view || strcmp(directory, g_get_home_dir()));
+  enabled = (!limited_view || !ishome);
   if (btnGoUp)
-    gtk_widget_set_sensitive(btnGoUp,enabled);
+    gtk_widget_set_sensitive(btnGoUp, enabled);
   set_active_item(directory);  
 
   /* some hacks to handle nasty smb urls */
@@ -1521,6 +1743,17 @@ browse_directory (gchar *directory)
   if (current_directory) 
     g_free(current_directory);
   current_directory = g_strdup (directory);
+  
+  if (!ishome)
+    {
+       if (strlen(strrchr(directory, '/')) > 1)
+         msg = g_strdup_printf("%s %s",_("Opening"), strrchr(directory, '/') + 1);
+       else /* should only happen for "/" */
+         msg = g_strdup_printf("%s %s",_("Opening"), strrchr(directory, '/'));
+       gpe_popup_infoprint(GDK_DISPLAY(), msg);
+       g_free(msg);
+    }
+  
   add_history (g_strdup(directory));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), directory);
   
@@ -1643,8 +1876,11 @@ view_list (GtkWidget *widget)
 
 
 static gboolean
-tree_button_press (GtkWidget *tree,GdkEventButton *b, gpointer user_data)
+tree_button_press (GtkWidget *tree, GdkEventButton *b, gpointer user_data)
 {
+  if (b->button == 1)
+    gtk_widget_grab_focus(tree);
+  
   if ((b->button == 3) || ((b->button == 1) && (b->type == GDK_2BUTTON_PRESS)))
     {
       gint x, y;
@@ -1679,61 +1915,47 @@ tree_button_press (GtkWidget *tree,GdkEventButton *b, gpointer user_data)
         show_popup (NULL, NULL);
   }
 
-  return TRUE;
+  return FALSE;
 }
 
 
 static gboolean
-tree_button_release (GtkWidget *tree, GdkEventButton *b)
+tree_focus_in (GtkWidget *tree, GdkEventFocus *f, gpointer data)
 {
-  
-  if (b->button == 1)
-    {
-      gint x, y;
-      GtkTreeViewColumn *col;
-      GtkTreePath *path;
-  
-      x = b->x;
-      y = b->y;
-
-      if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree),
-					 x, y,
-					 &path, &col,
-					 NULL, NULL))
-      {
-	    /*GtkTreeIter iter;
-        GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
-	    FileInformation *i;
-	    gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
-        */
-        gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree), path, NULL, FALSE);
-
-	   // gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, tree == view_widget ? COL_DATA : COL_DIRDATA, &i, -1);
-		
-	    gtk_tree_path_free (path);
-		
-        //button_clicked (NULL, i);
-      }
-  }
-  return TRUE;
+  active_view = tree;
+  return FALSE;
 }
 
 
+static void
+setup_tabchain(void)
+{
+  GList *chain = NULL;
+  
+  chain = g_list_append(chain, main_paned);
+  gtk_container_set_focus_chain(GTK_CONTAINER(vbox), chain);
+  g_list_free(chain);
+}
+
 static GtkWidget*
 create_view_widget_list(void)
 {
     GtkWidget *treeview;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+    GtkTreeSelection *selection;
 	
     if (GTK_IS_WIDGET(view_widget)) gtk_widget_destroy(view_widget);
     if (GTK_IS_WIDGET(view_window)) gtk_widget_destroy(view_window);
 	
 	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
-  	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview),TRUE);
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(treeview),TRUE);
-	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview),TRUE);
-  
+  	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview), FALSE);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview), TRUE);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+    
+    
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	g_object_set(renderer,"stock-size",GTK_ICON_SIZE_SMALL_TOOLBAR,NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("Icon"),
@@ -1778,9 +2000,9 @@ create_view_widget_list(void)
 
 	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
 		G_CALLBACK(tree_button_press), NULL);
-	g_signal_connect (G_OBJECT(treeview), "button_release_event", 
-		G_CALLBACK(tree_button_release), NULL);
-	
+	g_signal_connect (G_OBJECT(treeview), "focus-in-event", 
+		G_CALLBACK(tree_focus_in), NULL);
+
     gtk_widget_show_all(view_window);
 	return treeview;
 }
@@ -1823,12 +2045,15 @@ create_dir_view_widget(void)
     GtkWidget *treeview;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+    GtkTreeSelection *selection;
 	
 	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (dirstore));
-  	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview),FALSE);
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(treeview),TRUE);
-	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview),TRUE);
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview),TRUE);
+  	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview), FALSE);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), TRUE);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
   
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	g_object_set(renderer,"stock-size",GTK_ICON_SIZE_SMALL_TOOLBAR,NULL);
@@ -1860,9 +2085,8 @@ create_dir_view_widget(void)
     
 	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
 		G_CALLBACK(tree_button_press), NULL);
-	g_signal_connect (G_OBJECT(treeview), "button_release_event", 
-		G_CALLBACK(tree_button_release), NULL);
-	
+	g_signal_connect (G_OBJECT(treeview), "focus-in-event", 
+		G_CALLBACK(tree_focus_in), NULL);
     gtk_widget_show_all(dir_view_window);
 	return treeview;
 }
@@ -1934,53 +2158,63 @@ main (int argc, char *argv[])
 
   /* main menu */ 
   mMain = create_mMain(window);
-  gtk_box_pack_start(GTK_BOX(vbox),mMain,FALSE,TRUE,0);
+  gtk_box_pack_start(GTK_BOX(vbox), mMain, FALSE, TRUE, 0);
   
   /* location stuff */
   combo = gtk_combo_new ();
+  GTK_WIDGET_UNSET_FLAGS(combo, GTK_CAN_FOCUS);
   combo_signal_id = gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry),
-    "activate", GTK_SIGNAL_FUNC (goto_directory), NULL);
+                                        "activate", 
+                                        GTK_SIGNAL_FUNC (goto_directory), NULL);
   
   view_widget = create_view_widget_list();
-#warning necessary?  
+  
   dir_view_widget = create_dir_view_widget();
   
   storage_menu = build_storage_menu(directory_browser);
-  
+  GTK_WIDGET_UNSET_FLAGS(storage_menu, GTK_CAN_FOCUS);
+
   toolbar = gtk_toolbar_new ();
+  GTK_WIDGET_UNSET_FLAGS(toolbar, GTK_CAN_FOCUS);
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), GTK_ORIENTATION_HORIZONTAL);
 
   toolbar2 = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar2), GTK_ORIENTATION_HORIZONTAL);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_BACK,
-			    _("Go back in history."), NULL,
-			    G_CALLBACK (history_back), NULL, -1);
+  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_BACK,
+		                         _("Go back in history."), NULL,
+			                     G_CALLBACK (history_back), NULL, -1);
+  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_FORWARD,
-			    _("Go forward in history."), NULL,
-			    G_CALLBACK (history_forward), NULL, -1);
+  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_FORWARD,
+			                     _("Go forward in history."), NULL,
+			                     G_CALLBACK (history_forward), NULL, -1);
+  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
 
   btnGoUp = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_UP,
 			                          _("Go up one level."), NULL,
 			                          G_CALLBACK (up_one_level), NULL, -1);
+  GTK_WIDGET_UNSET_FLAGS(btnGoUp, GTK_CAN_FOCUS);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
   if (!directory_browser)
     {
-      gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_HOME,
-	                            _("Goto your home directory."), NULL,
-                                G_CALLBACK (set_directory_home), NULL, -1);
+      pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_HOME,
+	                                 _("Goto your home directory."), NULL,
+                                     G_CALLBACK (set_directory_home), NULL, -1);
+      GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
     }
                 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_REFRESH,
-			    _("Refresh current directory."), NULL,
-			    G_CALLBACK (refresh_current_directory), NULL, -1);
+  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_REFRESH,
+                                 _("Refresh current directory."), NULL,
+                                 G_CALLBACK (refresh_current_directory), NULL, -1);
+  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);    
                 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_STOP,
-			    _("Stop the current process."), NULL,
-			    G_CALLBACK (safety_check), NULL, -1);
+  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_STOP,
+                                 _("Stop the current process."), NULL,
+                                 G_CALLBACK (safety_check), NULL, -1);
+  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
@@ -1989,6 +2223,7 @@ main (int argc, char *argv[])
   btnIconView = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Icon view"), 
 			   _("View files as icons"), NULL, pw, 
 			   G_CALLBACK (view_icons), NULL);
+  GTK_WIDGET_UNSET_FLAGS(btnIconView, GTK_CAN_FOCUS);
   
   p = gpe_find_icon ("list-view");
   pw = gtk_image_new_from_pixbuf(p);
@@ -1996,13 +2231,15 @@ main (int argc, char *argv[])
 			   _("View files in a list"), NULL, pw, 
 			   G_CALLBACK (view_list), NULL);
   gtk_widget_set_sensitive(btnListView,FALSE);
+  GTK_WIDGET_UNSET_FLAGS(btnListView, GTK_CAN_FOCUS);
                
   pw = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, 
                                 gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar2)));
-  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Go!"), 
-			   _("Goto Location"), NULL, pw, 
-			   G_CALLBACK (goto_directory), NULL);
-  
+  pw = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Go!"), 
+                                _("Goto Location"), NULL, pw, 
+                                G_CALLBACK (goto_directory), NULL);
+  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
+
   gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (vbox));
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
   if (directory_browser)
@@ -2024,6 +2261,7 @@ main (int argc, char *argv[])
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(goto_menu_item), storage_menu);
   
   bluetooth_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Send via Bluetooth");
+  irda_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Send via Infrared");
   copy_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Copy");
   paste_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Paste");
   open_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Open With");
@@ -2055,6 +2293,7 @@ main (int argc, char *argv[])
     }
   else
 	gtk_widget_hide(dir_view_window);
+  gtk_widget_grab_focus(view_widget);
 
   if (argc < 2)
     set_directory_home (NULL);
@@ -2072,6 +2311,8 @@ main (int argc, char *argv[])
         set_directory_home (NULL);
     }
   
+  setup_tabchain();
+  do_scheduled_update();
   initialized = TRUE;  
   gtk_main();
 
